////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : CliqueHeuristicsSeparator.hh
//  Description      : Separate the clique inequalities with heuristics.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Tue Apr 04 14:07:11 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#ifndef CLIQUE_HEURISTICS_SEPARATOR_HH
#define CLIQUE_HEURISTICS_SEPARATOR_HH

#include <abacus/separator.h>

class Graph;
class StableSetLPSolution;


/**
 * This class provides an heuristic separation procedure for maximAL clique 
 * constraints.
 */
class CliqueHeuristicsSeparator: 
			   public ABA_SEPARATOR <ABA_CONSTRAINT, ABA_VARIABLE> {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *solution Pointer to the current LP-solution to be separated.
     * @param *theGraph Pointer to the graph of the problem instance.
     */
    CliqueHeuristicsSeparator(StableSetLPSolution *solution,
    			      Graph *theGraph);

    /**
     * Destructor.
     */
    ~CliqueHeuristicsSeparator();

    /**
     * This method provides the separation procedure itself and overrides the
     * pure virtual function of ABA_SEPARATOR.
     *
     * @param void
     * @return void
     */
    virtual void separate();


private:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p r i v a t e ) --------------------------
    // -------------------------------------------------------------------------

    /**
     * Get random number.
     *
     * @param bound Upper bound of random number.
     * @return Random number.
     */
    int getRandomNumber(int bound) const;


    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * This is a pointer to the current LP solution.
     */
    StableSetLPSolution *lpSolution;

    /**
     * Pointer to the graph of the problem instance.
     */
    Graph *itsGraph;

};

#endif


