////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : LocalCutSeparator.hh
//  Description      : Separate a local cut.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 14:18:09 2006 
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#ifndef LOCAL_CUT_SEPARATOR_HH
#define LOCAL_CUT_SEPARATOR_HH

#include <abacus/separator.h>
#include <vector>

using namespace std;

class Graph;
class StableSetLPSolution;
class StableSetMaster;



/**
 * Separate a local cut.
 */
class LocalCutSeparator: public ABA_SEPARATOR <ABA_CONSTRAINT, ABA_VARIABLE> {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *master Pointer to the master of the problem.
     * @param *solution Pointer to the current LP-Solution to be separated.
     * @param *theGraph Pointer to the graph of the problem instance.
     */
    LocalCutSeparator(ABA_MASTER *master, StableSetLPSolution *solution,
    		      Graph *theGraph);

    /**
     * Destructor.
     */
    ~LocalCutSeparator();

    /**
     * This method provides the separation procedure itself and overrides the
     * pure virtual function of ABA_SEPARATOR.
     *
     * @param void
     * @return void
     */
    virtual void separate();


private:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p r i v a t e ) --------------------------
    // -------------------------------------------------------------------------

    /**
     * Compute a small subgraph.
     *
     * @param void
     * @return void
     */
    bool getPartitialGraph();

    /**
     * Calculate greates common dividor.
     *
     * @param a First number.
     * @param b Second number.
     * @return gcd(a,b).
     */
    double calculateGCD(double a, double b);

    /**
     * The function stableSetMaster().
     *
     * @param void
     * @return Pointer to a StableSetMaster.
     */
    StableSetMaster *stableSetMaster();


    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * Pointer to the master of the problem.
     */
    ABA_MASTER *master_;

    /**
     * This is a pointer to the current LP solution.
     */
    StableSetLPSolution *lpSolution;

    /**
     * Pointer to the graph of the problem instance.
     */
    Graph *itsGraph;

    /**
     * Mamximal number of nodes of the subgraph.
     */
    const static int MAX_NUMBER_OF_NODES = 15;

    /**
     * Minimum number of nodes of the subgraph.
     */
    const static int MIN_NUMBER_OF_NODES = 4;

    /**
     * Number of nodes of the subgraph.
     */
    int numberOfNodes;

    /**
     * The nodes of the subgraph.
     */
    vector<int> nodes;

    /**
     * Adjacent list of the subgraph.
     */
    vector< vector<int> > adjacentList;

};

#endif


