////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : MaxCliquesSeparator.hh
//  Description      : Exact clique separation.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 14:47:59 2006
//  Last modified by : -
//  Last modified on : - 
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#ifndef MAX_CLIQUES_SEPARATOR_HH
#define MAX_CLIQUES_SEPARATOR_HH

#include <abacus/separator.h>
#include <vector>

using namespace std;

class Clique;
class Graph;
class StableSetLPSolution;
class StableSetMaster;



/**
 * This class provides an exact separation procedure for max clique constraints.
 */
class MaxCliquesSeparator: public ABA_SEPARATOR <ABA_CONSTRAINT, ABA_VARIABLE> {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *master Pointer to the master of the problem.
     * @param *solution Pointer to the current LP-Solution to be separated.
     * @param *theGraph Pointer to the graph of the problem instance.
     * @param *clique Pointer to the class Clique.
     */
    MaxCliquesSeparator(ABA_MASTER *master, StableSetLPSolution *solution,
    			Graph *theGraph, Clique *clique);

    /**
     * Destructor.
     */
    ~MaxCliquesSeparator();

    /**
     * This method provides the separation procedure itself and overrides the
     * pure virtual function of ABA_SEPARATOR.
     */
    virtual void separate();


private:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p r i v a t e ) --------------------------
    // -------------------------------------------------------------------------

    /**
     * Compute the next max clique.
     *
     * @param *nodeOfClique Pointer to the clique which is increased to
     *         a maximal one.
     * @return true Max clique computed.
     * @return false All maximal cliques have been found.
     */
    bool getNextClique(vector<int> *nodesOfClique) const;

    /**
     * The function stableSetMaster().
     *
     * @param void
     * @return Pointer to the corresponding object of the class
     *         StableSetMaster.
     */
    StableSetMaster *stableSetMaster();


    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * Pointer to the master of the problem.
     */
    ABA_MASTER *master_;

    /**
     * This is a pointer to the current LP solution.
     */
    StableSetLPSolution *lpSolution;

    /**
     * Pointer to the graph of the problem instance.
     */
    Graph *itsGraph;

    /**
     * Pointer to the class Clique which handles the storage of all maximal
     * cliques.
     */
    Clique *maxCliques;

};

#endif


