////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : Node.cpp
//  Description      : The variable.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       :
//  Last modified by :
//  Last modified on : 
//  Update count     :
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#include "abacus/master.h"
#include "Node.hh"



// -----------------------------------------------------------------------------
// C o n s t r u c t o r
//
// The main purpose of this constructor is to initialize an ABA_VARIABLE. It
// calls the contructor of this class with several arguments:
// 1.) A pointer to the corresponding master of the optimization
// 2.) A pointer to the subproblem associated with this variable. Because the
//     variables for each node are independent of a sub problem, the pointer
//     is a 0-pointer.
// 3.) Do not delete this variable from the set of active variables.
// 4.) The variables are not local valid, but global.
// 5.) The objective function coefficient
// 6.) The lower bound of the variable is 0.
// 7.) The upper bound of the variable is 1.
// 8.) The Variable is of binary type.
// -----------------------------------------------------------------------------
Node::Node(ABA_MASTER *master, int nodesNumber, double objectiveCoefficient):
   ABA_VARIABLE(master, 0, false, false, objectiveCoefficient,
                 0.0, 1.0, ABA_VARTYPE::Binary),
   numberOfNode(nodesNumber)
{
}


// -----------------------------------------------------------------------------
// D e s t r u c t o r
// -----------------------------------------------------------------------------
Node::~Node() {
}


// -----------------------------------------------------------------------------
// n o d e N u m b e r
//
// Receive the number of the node corresponding to this variable.
// -----------------------------------------------------------------------------
int Node::nodeNumber() const {
    return numberOfNode;
}


