////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : Node.hh
//  Description      : The variable.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 14:53:39 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#ifndef NODE_HH
#define NODE_HH


#include "abacus/variable.h"


/**
 * The class Node is derived from the abstract class ABA_Variable.
 * Each node of the graph is identified with a variable in the integer 
 * programming formulation of the Stable Set problem. Therefore,
 * each member of this class is a variable.
 */
class Node: public ABA_VARIABLE {


public:
    
    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------    
    
    /**
     * Constructor.
     *
     * @param *master Pointer to the corresponding master of the optimization.
     * @param nodesNumber The number of a node to identify the variable.
     * @param objectiveCoefficient The objective function coefficient of the 
     *                             variable.
     */
    Node(ABA_MASTER *master, int nodesNumber, double objectiveCoefficient);
    
    /**
     * Destructor.
     */
    virtual ~Node();
    
    /** 
     * Receive node number.
     *
     * @param void
     * @return Number of the node corresponding to this variable.
     */
    int nodeNumber() const;
    
    
private:
    
    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------    
          
    /**
     * The number of the node corresponding to this variable.
     */
    int numberOfNode;
};

#endif


