////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : Preprocessing.hh
//  Description      : Preprocessing for the maximum stable set problem.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Thu Apr 06 10:08:39 2006
//  Last modified by : -
//  Last modified on : - 
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#ifndef PREPROCESSING_HH
#define PREPROCESSING_HH


//#include "abacus/constraint.h"
//#include <vector>

//using namespace std;


class Graph;
class StableSetMaster;
class StableSetStatistics;



/**
 * The class 
 */
class Preprocessing {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *master Pointer to the corresponding master of the optimization.
     * @param *theGraph Pointer to the object representing the graph of the
     *                  problem.
     * @param *theStatistics Pointer to an object managing the statistics.
     */
    Preprocessing(StableSetMaster *master, Graph *theGraph,
		  StableSetStatistics *theStatistics);

    /**
     * Destructor.
     */
    virtual ~Preprocessing();

    /**
     * Start preprocessing.
     *
     * @param void
     * @return void
     */
    void preprocessing();


private:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p r i v a t e ) --------------------------
    // -------------------------------------------------------------------------

    /**
     * Try to fix nodes according to cliques.
     *
     * @param void
     * @return True If at least one node could be fixed and the graph is not
     *              empty.
     * @return False No fixing could be done or the graph is empty.
     */
    bool fixVariablesClique();

    /**
     * Solve the initial LP containing only the edge constraints
     * (next to the non-negativity constraints).
     *
     * @param void
     * @return Pointer to the optimal LP solution.
     */
    double* solveLP();

    /**
     * Fix variables which have value 1 in the LP solution.
     *
     * @param *sol Pointer to the optimal LP solution.
     * @return True If at least one node could be fixed.
     * @return False No fixing could be done.
     */
    bool fixVariablesEdgeLP(double *sol);


    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * Pointer to the stable set master.
     */
    StableSetMaster *itsMaster;

    /**
     * Store a pointer to the object representing the graph of the problem
     * instance.
     */
    Graph *itsGraph;

    /**
     * Pointer to the statistics.
     */
    StableSetStatistics *itsStatistics;
  
};

#endif




