////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : ProjectionNode.cpp
//  Description      : Node of the edge projection tree.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of L'Aquvila and
//                     University of Heidelberg
//  Created on       : Wed Apr 05 15:30:06 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#include "ProjectionNode.hh"


// -----------------------------------------------------------------------------
// --------------- M e t h o d s  ( p u b l i c ) ------------------------------
// -----------------------------------------------------------------------------


// -----------------------------------------------------------------------------
// C o n s t r u c t o r
//
// Copy all information of this node.
// -----------------------------------------------------------------------------
ProjectionNode::ProjectionNode(int *theEdge, vector<int> *theDeletedNodes,
                               vector<int*> *theDeletedEdges,
                               vector<int*> *theFalseEdges,
                               ProjectionNode *prevNode):
    previousNode(prevNode)
{
    
    int i;    // Loop variable.
    int size; // Store a size.

    //
    // Copy projection edge.            
    //
    edge[0] = theEdge[0];
    edge[1] = theEdge[1];

    //
    // Copy the deleted nodes.
    //
    if (theDeletedNodes != NULL) {
        if(!theDeletedNodes->empty()) {
        size = theDeletedNodes->size();
        deletedNodes.resize(size);
        for (i=0; i<size; i++) {
            deletedNodes[i] = (*theDeletedNodes)[i];
        }}
    }// if
    
    //
    // Copy the deleted edges.
    //
    if (!theDeletedEdges->empty()) {
        int *anEdge;
        size = theDeletedEdges->size();
        deletedEdges.resize(size);
        for (i=0; i<size; i++) {
            anEdge = new int[2];
            anEdge[0] = (*theDeletedEdges)[i][0];
            anEdge[1] = (*theDeletedEdges)[i][1];
            deletedEdges[i] = anEdge;
        }
    }// if
    
    //
    // Copy the false edges.
    //
    if (!theFalseEdges->empty()) {
        int *anEdge;
        size = theFalseEdges->size();
        falseEdges.resize(size);
        for (i=0; i<size; i++) {
            anEdge = new int[2];
            anEdge[0] = (*theFalseEdges)[i][0];
            anEdge[1] = (*theFalseEdges)[i][1];
            falseEdges[i] = anEdge;
        }
    }// if
    
}// end constructor


// -----------------------------------------------------------------------------
// D e s t r u c t o r
// -----------------------------------------------------------------------------
ProjectionNode::~ProjectionNode() {

    // Delete all edges.
    if (!deletedEdges.empty()) {
        for (int i=deletedEdges.size()-1; i>=0; i--) {
            delete [] deletedEdges[i];
            deletedEdges[i] = NULL;
        }
        deletedEdges.clear();
    }
    // Delete all false edges.
    if (!falseEdges.empty()) {
        for (int i=falseEdges.size()-1; i>=0; i--) {
            delete [] falseEdges[i];
            falseEdges[i] = NULL;
        }
        falseEdges.clear();
    }
    // Delete all sons of this node.
    if (!itsNodes.empty()) {
        for (int i=itsNodes.size()-1; i>=0; i--) {
            // Call destructor of each son of this node.
            delete itsNodes[i];
            itsNodes[i] = NULL;
        }
        itsNodes.clear();
    }

}// end destructor


// -----------------------------------------------------------------------------
// a d d N o d e
// -----------------------------------------------------------------------------
ProjectionNode* ProjectionNode::addNode(int theEdge[2],
                                        vector<int> *theDeletedNodes,
                                        vector<int*> *theDeletedEdges,
                                        vector<int*> *theFalseEdges) {
    
    ProjectionNode *pN;
    pN = new ProjectionNode(theEdge, theDeletedNodes, theDeletedEdges,
                            theFalseEdges, this);
    itsNodes.push_back(pN);  
    
    return pN;
}
    

// -----------------------------------------------------------------------------
// g e t E d g e
// -----------------------------------------------------------------------------
int ProjectionNode::getEdge(int i) {
    return edge[i];
}


// -----------------------------------------------------------------------------
// g e t D e l e t e d N o d e s
// -----------------------------------------------------------------------------
vector<int>* ProjectionNode::getDeletedNodes() {  
    return &deletedNodes;
}


// -----------------------------------------------------------------------------
// g e t D e l e t e d E d g e s
// -----------------------------------------------------------------------------
vector<int*>* ProjectionNode::getDeletedEdges() {    
    return &deletedEdges;
}

    
// -----------------------------------------------------------------------------
// g e t F a l s e E d g e s
// -----------------------------------------------------------------------------
vector<int*>* ProjectionNode::getFalseEdges() {    
    return &falseEdges;
}


// -----------------------------------------------------------------------------
// g e t P r e v i o u s N o d e
// -----------------------------------------------------------------------------
ProjectionNode* ProjectionNode::getPreviousNode() {    
    return previousNode;
}


