////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : ShortestPathCalculator.hh
//  Description      :
//  Author           : Marcus Oswald
//  Email            : Marcus.Oswald@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Jul 15 11:00:50 2002
//  Last modified by : Rebennack
//  Last modified on : 06.10.2005
//  Update count     : 20
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#ifndef SHORTEST_PATH_CALCULATOR_HH
#define SHORTEST_PATH_CALCULATOR_HH



/**
 * This class realizes the exact calculation of shortest paths by dijkstra with
 * heaps.
 */
class ShortestPathCalculator {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param nNodes Number of nodes.
     * @param nEdges Number of edges.
     * @param *adjPtr Pointer of indizes to the adjaznes list. It is the start
     *		     index to all the adjazent node.
     * @param *adjList  Pointer of adjazens list.
     * @param *edgeWeight
     * @param bound
     * @param tol
     */
    ShortestPathCalculator(int nNodes, int nEdges, int* adjPtr, int* adjList,
    			   double* edgeWeight, double bound, double tol);

    /**
     * Destructor.
     */
    ~ShortestPathCalculator();

    /**
     * Calculate the shortest path.
     *
     * @param startNode Node to start the path.
     * @param endNode Node to end the path.
     * @param *predNode Nodes of the path.
     *
     * @return Weight of shortes path.
     */
    double calculateShortestPath(int startNode, int endNode, int* predNode);


private:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p r i v a t e ) --------------------------
    // -------------------------------------------------------------------------

    /**
     * Build heap.
     *
     * @param startNode
     */
    void buildHeap(int startNode);

    /**
     *
     *
     * @return
     */
    int extractMin();

    /**
     *
     *
     * @param node
     * @param label
     */
    void changeKey(int node, double label);


    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * Number of nodes of the graph.
     */
    int m_nNodes;

    /**
     * Number of edges of the graph.
     */
    int m_nEdges;

    /**
     *
     */
    int* m_adjPtr;

    /**
     *
     */
    int* m_adjList;

    /**
     *
     */
    double* m_edgeWeight;

    /**
     *
     */
    double m_bound;

    /**
     *
     */
    double m_tol;

    /**
     *
     */
    int m_heapSize;

    /**
     *
     */
    int* m_heap;

    /**
     *
     */
    int* m_invHeap;

    /**
     *
     */
    double* m_nodeLabel;

};// end: class shortestPathCalculator

#endif


