////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : StableSetBranchRule.cpp
//  Description      : 
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 16:48:06 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#include "abacus/master.h"
#include "StableSetBranchRule.hh"
#include "StableSetSub.hh"



// -----------------------------------------------------------------------------
// C o n s t r u c t o r
// -----------------------------------------------------------------------------
StableSetBranchRule::StableSetBranchRule(ABA_MASTER *master, int node,
					 vector<int> *nodes):
    ABA_BRANCHRULE(master),
    master_(master),
    setToOne(node)
{

    setToZeroSize = nodes->size();

    if (setToZeroSize > 0) {
	setToZero = new int[setToZeroSize];
	for (int i=0; i<setToZeroSize; i++) {
	    setToZero[i] = (*nodes)[i];
	}
    }
}


// -----------------------------------------------------------------------------
// D e s t r u c t o r
// -----------------------------------------------------------------------------
StableSetBranchRule::~StableSetBranchRule() {

    if (setToZeroSize > 0) {
	delete [] setToZero;
    }

}


// -----------------------------------------------------------------------------
// e x t r a c t
//
// Activate this node of the branching tree.
// -----------------------------------------------------------------------------
int StableSetBranchRule::extract(ABA_SUB *sub) {

    int  i, j;
    int  size;

    ((StableSetSub *)sub)->setVertex(setToOne, true);


    for (i=0; i<setToZeroSize; i++) {
	// Check for contradictions.
	if(sub->variable(setToZero[i])->fsVarStat()->set()) {
	    return 1;
	}
	((StableSetSub *)sub)->setVertex(setToZero[i], false);
    }

    return 0;
}


