////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : StableSetBranchRule.hh
//  Description      :
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 16:49:46 2006
//  Last modified by : -
//  Last modified on : -  
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#ifndef STABLE_SET_BRANCH_RULE_HH
#define STABLE_SET_BRANCH_RULE_HH


#include "abacus/branchrule.h"
class ABA_MASTER;


#include <vector>

using namespace std;



/**
 * This class is derived from tha abacus class ABA_BRANCHRULE. It 
 * represents one node of the Barnch & Cut tree.
 */
class StableSetBranchRule: public ABA_BRANCHRULE {


public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *master Pointer to the corresponding master of the optimization.
     * @param node The node which is set to value 1.
     * @param *nodes Pointer to a vector storing the nodes to be set to value 0.
     */
    StableSetBranchRule(ABA_MASTER *master, int node, vector<int> *nodes);

    /**
     * Destructor.
     */
    virtual ~StableSetBranchRule();

    /**
     * Activate this branching node.
     *
     * @param *sub Pointer to the current subproblem.
     * @return 0 Setting of the variables could be done without error.
     * @return 1 There was an error.
     */
    virtual int extract(ABA_SUB *sub);


private:

    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * Pointer to the master of the problem.
     */
    ABA_MASTER *master_;

    /**
     * Node which is set to value 1.
     */
    int setToOne;

    /**
     * Set of nodes which are fixed to value 0.
     */
    int* setToZero;

    /**
     * Number of nodes which are set to 0.
     */
    int setToZeroSize;

};// end: class StableSetBranchRule

#endif




