////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : StableSetLPSolution.cpp
//  Description      : LP solution to the current subproblem.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 16:52:35 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#include "StableSetLPSolution.hh"
#include "Graph.hh"
#include "StableSetMaster.hh"
#include "StableSetSub.hh"



// -----------------------------------------------------------------------------
// C o n s t r u c t o r
// -----------------------------------------------------------------------------
StableSetLPSolution::StableSetLPSolution (ABA_MASTER *master, ABA_SUB *sub,
					  int numberOfNodes, double *xVal):
    ABA_LPSOLUTION<ABA_VARIABLE,ABA_CONSTRAINT> (master),
    m_sub(sub)
{
    // Number of nodes.
    nVarCon_ = numberOfNodes;

    // Initialize zVal_.
    zVal_.realloc(numberOfNodes);
    for (int i=0; i<nVarCon_; i++) {
        zVal_[i] = xVal[i];
    }
}


// -----------------------------------------------------------------------------
// D e s t r u c t o r
// -----------------------------------------------------------------------------
StableSetLPSolution::~StableSetLPSolution() {
}


// -----------------------------------------------------------------------------
// s t a b l e S e t M a s t e r
//
// Returns pointer of the master.
// -----------------------------------------------------------------------------
StableSetMaster *StableSetLPSolution::stableSetMaster() const {
    return (StableSetMaster*)master_;
}


// -----------------------------------------------------------------------------
// s u b
//
// Return pointer to the current subproblem.
// -----------------------------------------------------------------------------
ABA_SUB *StableSetLPSolution::sub() const {
    return (ABA_SUB*)m_sub;
}





