////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : StableSetLPSolution.hh
//  Description      : LP solution to the current subproblem.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Wed Apr 05 16:56:23 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#ifndef STABLE_SET_LP_SOLUTION_HH
#define STABLE_SET_LP_SOLUTION_HH

#include "abacus/lpsolution.h"

class StableSetMaster;


/**
 * This class stores the last LP-solution and provides it to the separation
 * routines.
 */
class StableSetLPSolution: public ABA_LPSOLUTION <ABA_VARIABLE,ABA_CONSTRAINT> {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *master Pointer to the corrsponding master of the optimization.
     * @param *sub Pointer to the current suproblem.
     * @param numberOfNodes Number of nodes of the graph.
     * @param *xVal Pointer which stores the current LP solution values.
     */
    StableSetLPSolution(ABA_MASTER *master, ABA_SUB*sub, int numberOfNodes,
    			double *xVal);

    /**
     * Destructor.
     */
    ~StableSetLPSolution();

    /**
     * Get stable set master.
     *
     * @param void
     * @return Pointer to the stable set master.
     */
    StableSetMaster *stableSetMaster() const;

    /**
     * Return the current subproblem.
     *
     * @param void
     * @return Pointer to the current subproblem.
     */
    ABA_SUB *sub() const;


private:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p r i v a t e ) --------------------------
    // -------------------------------------------------------------------------
    
    /**
     * Pointer to the current suprobelm.
     */
    ABA_SUB *m_sub;

};

#endif


