////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : StableSetMain.cpp
//  Description      : Call stable set solver.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Thu Apr 06 08:16:36 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#include "StableSetMaster.hh"
#include <iostream>

using namespace std;


/**
 * Call the stable set solver. Check each generated inequality if it is valid.
 */
int main(int argc, char *argv[]) {


    // local variables
    #ifdef ABACUS_VISUAL_CPP
        char inputFileName[100];
	char inequalitiesFileName[100];
        char smallGraphFileName[100];
    #else
        char *inputFileName;
	char *inequalitiesFileName;
        char *smallGraphFileName;
    #endif

    // Read input parameter
    #ifdef ABACUS_VISUAL_CPP
        //specific the input file name
        cout << "input file: ";
        cin >> inputFileName;
	cout << "inequalities file: ";
	cin >> inequalitiesFileName;
	cout << "small graph file: ";
	cin >> smallGraphFileName;
    #else
        // analyze command line arguments
        if (argc != 4){
            cerr << "usage: " << argv[0] << " <instance> <inequalities>";
            cerr << " <smallGraphName>";
	    cerr << endl;
            return 1;
        }
        inputFileName = argv[1];
	inequalitiesFileName = argv[2];
        smallGraphFileName = argv[3];
    #endif

    //
    // Generate a stable set and optimize...
    //
    StableSetMaster *stableSet;
    stableSet = new StableSetMaster(inputFileName, inequalitiesFileName,//
				    smallGraphFileName);  
    ABA_MASTER::STATUS status = stableSet->optimize();

    // Clean up.
    delete stableSet;


    cout << "\n\nCheck inequalities:" << endl;
    cout << "----------------------------------------------------";
    cout << "------------------------------------------" << endl;

    //
    // Check all inequalities for their validity.
    //
    string check("./CheckInequalities.exe ");
    check.append("~/Diplomarbeit/Abacus/Stable_Set/Instanzen/");
    check.append(inputFileName);
    check.append(" ");
    check.append(inequalitiesFileName);

    // Call program to check all inequalities.
    int notValid = system(check.c_str());
    check.clear(); 

    cout << "----------------------------------------------------";
    cout << "------------------------------------------\n" << endl;

    if(status || notValid) {
        cout << "\nNOT OPTIMAL." << endl;
        return 1;
    }
    else {
        cout << "Optimality profen.\n" << endl;
	return 0;
    }
}


