////////////////////////////////////// c++ ////////////////////////////////////
//
//  Module           : StableSetModKSeparator.hh
//  Description      : Mod-k separation.
//  Author           : Hanna Seitz, Steffen Rebennack
//  Email            : Hanna.Seitz@informatik.uni-heidelberg.de;
//                     srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : University of Heidelberg
//  Created on       : Thu Apr 06 10:51:17 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
///////////////////////////////////////////////////////////////////////////////


#ifndef STABLE_SET_MOD_2_SEPARATOR_HH
#define STABLE_SET_MOD_2_SEPARATOR_HH

#include <abacus/separator.h>

class Graph;
class StableSetLPSolution;


/**
 * This class realizes the separation of metric (triangle) - inequalities. 
 */
class StableSetModKSeparator:public ABA_SEPARATOR<ABA_CONSTRAINT, ABA_VARIABLE> {

public:

    // -------------------------------------------------------------------------
    // ------------- M e t h o d s  ( p u b l i c ) ----------------------------
    // -------------------------------------------------------------------------

    /**
     * Constructor.
     *
     * @param *lpSolution Pointer to the current LP-Solution to be separated.
     * @param *theGraph Pointer to the graph.
     */
    StableSetModKSeparator(StableSetLPSolution *solution,
			   Graph *itsGraph);

    /**
     * Destructor.
     */
    ~StableSetModKSeparator();

    /**
     * Contains the separation procedure. Overrides the pure virtual function of
     * ABA_SEPARATOR.
     *
     * @param void  
     * @return void
     */
    virtual void separate();

    /**
     * Rank the inequalities.
     *
     * @param *rank Pointer to the ranking of the inequalities.  
     * @return void
     */
    void applyRanking(ABA_BUFFER<double> *rank);


private:

    // -------------------------------------------------------------------------
    // ------------- V a r i a b l e s  ( p r i v a t e ) ----------------------
    // -------------------------------------------------------------------------

    /**
     * Pointer to the current lp solution.
     */
    StableSetLPSolution *lpSolution;

    /**
     * Number of variables.
     */
    int nVariables;

    /**
     * Pointer to the corresponding graph of the problem instance of class
     * StableSetMaster.
     */
    Graph *itsGraph;

};

#endif


