////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : StableSetStatistics.cpp
//  Description      : Statistics.
//  Author           : Steffen Rebennack
//  Email            : srebenac@ix.urz.uni-heidelberg.de;
//                     steffen.rebennack@web.de
//  Copyright        : (C) 2006 by the University of Heidelberg
//  Created on       : Thu Apr 06 11:12:50 2006
//  Last modified by : -
//  Last modified on : -
//  Update count     : 0
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////

#include "StableSetStatistics.hh"

#include <iostream>
using namespace std;

// -----------------------------------------------------------------------------
// --------------- M e t h o d s  ( p u b l i c ) ------------------------------
// -----------------------------------------------------------------------------


// -----------------------------------------------------------------------------
// C o n s t r u c t o r
//
// Initialize all variables.
// -----------------------------------------------------------------------------
StableSetStatistics::StableSetStatistics():
    edgeInequalities(0),
    poolSeparationCuts(0),
    maxCliquesMemoryCuts(0),
    oddCyclesCuts(0),
    liftedOddCyclesCuts(0),
    exactCliquesCuts(0),
    cliqueHeuristicsCuts(0),
    edgeProjectionCuts(0),
    localCutCuts(0),
    modKCuts(0),
    dualBoundOfRoot(0),
    largeCuts(0),
    roundedSolutions(0),
    improvedSolutions(0),
    solutionFoundBy(-1),
    alphaRoot(-1)
{
}


// -----------------------------------------------------------------------------
// D e s t r u c t o r
// -----------------------------------------------------------------------------
StableSetStatistics::~StableSetStatistics() {
}


// -----------------------------------------------------------------------------
// e d g e I n e q u a l i t i e s C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::edgeInequalitiesCut(int num) {
    edgeInequalities = num < 0 ? edgeInequalities : edgeInequalities + num;
}


// -----------------------------------------------------------------------------
// p o o l S e p a r a t i o n C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::poolSeparationCut(int num) {
    poolSeparationCuts = num < 0 ? poolSeparationCuts : poolSeparationCuts +num;
}


// -----------------------------------------------------------------------------
// m a x C l i q u e s M e m o r y C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::maxCliquesMemoryCut(int num) {
    maxCliquesMemoryCuts = num < 0 ? maxCliquesMemoryCuts
				   : maxCliquesMemoryCuts + num;
}


// -----------------------------------------------------------------------------
// o d d C y c l e s C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::oddCyclesCut(int num) {
    oddCyclesCuts = num < 0 ? oddCyclesCuts : oddCyclesCuts + num;
}


// -----------------------------------------------------------------------------
// l i f t e d O d d C y c l e s C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::liftedOddCyclesCut(int num) {
    liftedOddCyclesCuts = num < 0 ? liftedOddCyclesCuts
				  : liftedOddCyclesCuts + num;
}


// -----------------------------------------------------------------------------
// e x a c t C l i q u e s C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::exactCliquesCut(int num) {
    exactCliquesCuts = num < 0 ? exactCliquesCuts : exactCliquesCuts + num;
}


// -----------------------------------------------------------------------------
// c l i q u e H e u r i s t i c s C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::cliqueHeuristicsCut(int num) {
    cliqueHeuristicsCuts = num < 0 ? cliqueHeuristicsCuts 
				   : cliqueHeuristicsCuts + num;
}


// -----------------------------------------------------------------------------
// e d g e P r o j e c t i o n C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::edgeProjectionCut(int num) {
    edgeProjectionCuts = num < 0 ? edgeProjectionCuts
				 : edgeProjectionCuts + num;
}


// -----------------------------------------------------------------------------
// l o c a l C u t C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::localCutCut(int num) {
    localCutCuts = num < 0 ? localCutCuts : localCutCuts + num;
}

// -----------------------------------------------------------------------------
// m o d K C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::modKCut(int num) {
    modKCuts = num < 0 ? modKCuts : modKCuts + num;
}

// -----------------------------------------------------------------------------
// l a r g e C u t C u t
// -----------------------------------------------------------------------------
void StableSetStatistics::largeCutCut(int num) {
    largeCuts = num < 0 ? largeCuts : largeCuts + num;
}


// -----------------------------------------------------------------------------
// s e t D u a l B o u n d
// -----------------------------------------------------------------------------
void StableSetStatistics::setDualBound(double val) {
    dualBoundOfRoot = val;
}


// -----------------------------------------------------------------------------
// r o u n d i n g H e u r i s t i c s
// -----------------------------------------------------------------------------
void StableSetStatistics::roundingHeuristics() {
    roundedSolutions++;
}


// -----------------------------------------------------------------------------
// i m p r o v e m e n t H e u r i s t i c s
// -----------------------------------------------------------------------------
void StableSetStatistics::improvementHeuristics() {
    improvedSolutions++;
}


// -----------------------------------------------------------------------------
// s o l u t i o n F o u n d
// -----------------------------------------------------------------------------
void StableSetStatistics::solutionFound(int index) {
    solutionFoundBy = index;
}


// -----------------------------------------------------------------------------
// s e t A l p h a R o o t
// -----------------------------------------------------------------------------
void StableSetStatistics::setAlphaRoot(int alpha) {
    alphaRoot = alpha;
}


// -----------------------------------------------------------------------------
// g e t E d g e I n e q u a l i t i e s C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getEdgeInequalitiesCut() const {
    return edgeInequalities;
}


// -----------------------------------------------------------------------------
// g e t P o o l S e p a r a t i o n C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getPoolSeparationCut() const {
    return poolSeparationCuts;
}


// -----------------------------------------------------------------------------
// g e t M a x C l i q u e s M e m o r y C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getMaxCliquesMemoryCut() const {
    return maxCliquesMemoryCuts;
}

// -----------------------------------------------------------------------------
// g e t O d d C y c l e s C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getOddCyclesCut() const {
    return oddCyclesCuts;
}


// -----------------------------------------------------------------------------
// g e t L i f t e d O d d C y c l e s C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getLiftedOddCyclesCut() const {
    return liftedOddCyclesCuts;
}


// -----------------------------------------------------------------------------
// g e t E x a c t C l i q u e s C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getExactCliquesCut() const {
    return exactCliquesCuts;
} 


// -----------------------------------------------------------------------------
// g e t C l i q u e H e u r i s t i c s C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getCliqueHeuristicsCut() const {
    return cliqueHeuristicsCuts;
}


// -----------------------------------------------------------------------------
// g e t E d g e P r o j e c t i o n C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getEdgeProjectionCut() const {
    return edgeProjectionCuts;
}

// -----------------------------------------------------------------------------
// g e t M o d K C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getModKCut() const {
    return modKCuts;
}


// -----------------------------------------------------------------------------
// g e t L o c a l C u t C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getLocalCutCut() const {
    return localCutCuts;
}


// -----------------------------------------------------------------------------
// g e t L a r g e C u t
// -----------------------------------------------------------------------------
int StableSetStatistics::getLargeCut() const {
    return largeCuts;
}
 

// -----------------------------------------------------------------------------
// g e t R o u n d e d S o l u t i o n s
// -----------------------------------------------------------------------------
int StableSetStatistics::getRoundedSolutions() const {
    return roundedSolutions;
}

// -----------------------------------------------------------------------------
// g e t I m p r o v e d S o l u t i o n s
// -----------------------------------------------------------------------------
int StableSetStatistics::getImprovedSolutions() const {
    return improvedSolutions;
}


// -----------------------------------------------------------------------------
// g e t A l p h a R o o t 
// -----------------------------------------------------------------------------
int StableSetStatistics::getAlphaRoot() const {
   return alphaRoot;
}


// -----------------------------------------------------------------------------
// g e t D u a l B o u n d
// -----------------------------------------------------------------------------
double StableSetStatistics::getDualBound() const {
   return dualBoundOfRoot;
}


// -----------------------------------------------------------------------------
// s o l u t i o n W a s F o u n d B y
// -----------------------------------------------------------------------------
char *StableSetStatistics::solutionWasFoundBy() const {
    
    char *methods[] = {"integer solution", "rounding heuristics",
		       "improvement heuristics", "preprocessing"};

    if (solutionFoundBy < 0 || solutionFoundBy > 3) {
         return "-1";
    }

    return methods[solutionFoundBy];
}


